# 💬 Bubble — Bulletin Boards for Gemini

Bubble is a Gemini-based bulletin board system that can be summarized as a union of station.martinrue.com, Reddit, WordPress, and issue trackers like GitHub Issues. It is implemented as an extension module to [GmCapsule](https://git.skyjake.fi/gemini/gmcapsule).

Bubble has been designed to be useful for both individuals and communities. It has multiple objectives and target audiences:

* Someone could run Bubble in a manner similar to WordPress, as a Gemini-based CMS for their capsule. Every user on a Bubble server gets their own personal "u/" feed that is available both for Gemini subscriptions and Tinylog followers.
* A group of people can run Bubble as their own version of Station. Users' posts appear in the shared All Posts feed on the front page.
* Groups of people can set up their own moderated "s/" subspaces on a larger Bubble instance for topic-specific discussions, much like subreddits on Reddit. Users can freely create new Bubble subspaces. Subspaces have their own home pages and Gemini/Atom feeds.
* A developer of Gemini software can switch a subspace to issue tracking mode and link it with a Git repository. Issue tracking subspaces behave like GitHub Issues (and other Git forges on the web) in that posts have an open/closed status and they can cross-reference other issues in the tracker and commits in the Git repository.

## Requirements

* Python 3.6+ (same as GmCapsule)
* MariaDB database
* `mariadb` (MariaDB Python Connector module)
* `pytz` (backwards-compatible time zone library)

## Installation

1. Decide on the database user name and password that will be used by Bubble to access your database server.
2. Create a MariaDB database using your admin credentials:
```sql
CREATE DATABASE bubble CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
GRANT ALL PRIVILEGES ON bubble.* TO 'db_user'@'localhost' IDENTIFIED BY 'db_password';
```
3. Configure GmCapsule to load the Bubble extension. In the server's configuration file, add the Bubble source directory (where *50_bubble.py* can be found) to your `modules`:
```ini
modules = /home/username/src/bubble
```
4. Copy the example configuration from _example.ini_ to your GmCapsule configuration file. Edit the configuration and update the database parameters with the correct values:
```ini
db.host = localhost
db.port = 3306
db.user = db_user
db.password = db_password
db.name = bubble
```
5. Update the rest of the `[bubble]` configuration including the capsule name, icon, description, and code of conduct.
6. Email notifications require a working sendmail-compatible CLI tool. This feature can be disabled by leaving `email.cmd` empty.
```ini
email.cmd = /usr/sbin/sendmail
email.from = admin@example.com
```
7. An account called "admin" is created when the Bubble database is initialized on the first run of the server. However, it does not have any registered client certificates so it is impossible to log in as "admin". The `admin.certpass` config variable sets a temporary password using which you can register certificates for the admin account.
```ini
admin.certpass = mypassword
```

After these steps, you should be able to access the "admin" account and use Bubble itself to continue the capsule configuration.

## License

Bubble is licensed under [BSD-2-Clause](https://opensource.org/license/bsd-2-clause/).
